@@ -327,45 +327,11 @@ class VersionedObject(object):
     @classmethod
     def to_json_schema(cls):
         obj_name = cls.obj_name()
-        field_schemas = {key: field.get_schema()
-                         for key, field in cls.fields.items()}
-        required_fields = [name for name, schema in
-                           sorted(field_schemas.items())]
-        namespace_key = cls._obj_primitive_key('namespace')
-        name_key = cls._obj_primitive_key('name')
-        version_key = cls._obj_primitive_key('version')
-        data_key = cls._obj_primitive_key('data')
-        changes_key = cls._obj_primitive_key('changes')
-
         schema = {
             '$schema': 'http://json-schema.org/draft-04/schema#',
             'title': obj_name,
-            'type': 'object',
-            'properties': {
-                namespace_key: {
-                    'type': 'string'
-                },
-                name_key: {
-                    'type': 'string'
-                },
-                version_key: {
-                    'type': 'string'
-                },
-                changes_key: {
-                    'type': 'array',
-                    'items': {
-                        'type': 'string'
-                    }
-                },
-                data_key: {
-                    'type': 'object',
-                    'description': 'fields of %s' % (obj_name),
-                    'properties': field_schemas,
-                    'required': required_fields
-                },
-            },
-            'required': [namespace_key, name_key, version_key, data_key]
         }
+        schema.update(obj_fields.Object(obj_name).get_schema())
         return schema
 
     @classmethod
@@ -599,8 +565,13 @@ class VersionedObject(object):
                self._obj_primitive_key('version'): target_version,
                self._obj_primitive_key('data'): primitive}
         if self.obj_what_changed():
-            obj[self._obj_primitive_key('changes')] = list(
-                self.obj_what_changed())
+            # NOTE(cfriesen): if we're downgrading to a lower version, then
+            # it's possible that self.obj_what_changed() includes fields that
+            # no longer exist in the lower version.  If so, filter them out.
+            what_changed = self.obj_what_changed()
+            changes = [field for field in what_changed if field in primitive]
+            if changes:
+                obj[self._obj_primitive_key('changes')] = changes
         return obj
 
     def obj_set_defaults(self, *attrs):
@@ -636,7 +607,8 @@ class VersionedObject(object):
 
     def obj_what_changed(self):
         """Returns a set of fields that have been modified."""
-        changes = set(self._changed_fields)
+        changes = set([field for field in self._changed_fields
+                       if field in self.fields])
         for field in self.fields:
             if (self.obj_attr_is_set(field) and
                     isinstance(getattr(self, field), VersionedObject) and
@@ -874,6 +846,28 @@ class ObjectListBase(collections.Sequence):
                 changes.add('objects')
         return changes
 
+    def __add__(self, other):
+        # Handling arbitrary fields may not make sense if those fields are not
+        # all concatenatable. Only concatenate if the base 'objects' field is
+        # the only one and the classes match.
+        if (self.__class__ == other.__class__ and
+                list(self.__class__.fields.keys()) == ['objects']):
+            return self.__class__(objects=self.objects + other.objects)
+        else:
+            raise TypeError("List Objects should be of the same type and only "
+                            "have an 'objects' field")
+
+    def __radd__(self, other):
+        if (self.__class__ == other.__class__ and
+                list(self.__class__.fields.keys()) == ['objects']):
+            # This should never be run in practice. If the above condition is
+            # met then __add__ would have been run.
+            raise NotImplementedError('__radd__ is not implemented for '
+                                      'objects of the same type')
+        else:
+            raise TypeError("List Objects should be of the same type and only "
+                            "have an 'objects' field")
+
 
 class VersionedObjectSerializer(messaging.NoOpSerializer):
     """A VersionedObject-aware Serializer.
