@@ -127,6 +127,9 @@ class FieldType(AbstractFieldType):
     def stringify(self, value):
         return str(value)
 
+    def get_schema(self):
+        raise NotImplementedError()
+
 
 class UnspecifiedDefault(object):
     pass
@@ -237,6 +240,16 @@ class Field(object):
         else:
             return self._type.stringify(value)
 
+    def get_schema(self):
+        schema = self._type.get_schema()
+        schema.update({'readonly': self.read_only})
+        if self.nullable:
+            schema['type'].append('null')
+        default = self.default
+        if default != UnspecifiedDefault:
+            schema.update({'default': default})
+        return schema
+
 
 class String(FieldType):
     @staticmethod
@@ -255,6 +268,9 @@ class String(FieldType):
     def stringify(value):
         return '\'%s\'' % value
 
+    def get_schema(self):
+        return {'type': ['string']}
+
 
 class SensitiveString(String):
     """A string field type that may contain sensitive (password) information.
@@ -307,8 +323,15 @@ class Enum(String):
             raise ValueError(msg)
         return super(Enum, self).stringify(value)
 
+    def get_schema(self):
+        return {'enum': self._valid_values}
+
 
 class UUID(FieldType):
+
+    _PATTERN = (r'^[a-fA-F0-9]{8}-?[a-fA-F0-9]{4}-?[a-fA-F0-9]{4}-?[a-fA-F0-9]'
+                r'{4}-?[a-fA-F0-9]{12}$')
+
     @staticmethod
     def coerce(obj, attr, value):
         # FIXME(danms): We should actually verify the UUIDness here
@@ -330,10 +353,14 @@ class UUID(FieldType):
 
             return str(value)
 
+    def get_schema(self):
+        return {'type': ['string'], 'pattern': self._PATTERN}
+
 
 class MACAddress(FieldType):
 
-    _REGEX = re.compile(r'[0-9a-f]{2}(:[0-9a-f]{2}){5}$')
+    _PATTERN = r'^[0-9a-f]{2}(:[0-9a-f]{2}){5}$'
+    _REGEX = re.compile(_PATTERN)
 
     @staticmethod
     def coerce(obj, attr, value):
@@ -343,10 +370,14 @@ class MACAddress(FieldType):
                 return lowered
         raise ValueError(_LE("Malformed MAC %s"), value)
 
+    def get_schema(self):
+        return {'type': ['string'], 'pattern': self._PATTERN}
+
 
 class PCIAddress(FieldType):
 
-    _REGEX = re.compile(r'^[0-9a-f]{4}:[0-9a-f]{2}:[0-1][0-9a-f].[0-7]$')
+    _PATTERN = r'^[0-9a-f]{4}:[0-9a-f]{2}:[0-1][0-9a-f].[0-7]$'
+    _REGEX = re.compile(_PATTERN)
 
     @staticmethod
     def coerce(obj, attr, value):
@@ -356,23 +387,59 @@ class PCIAddress(FieldType):
                 return newvalue
         raise ValueError(_LE("Malformed PCI address %s"), value)
 
+    def get_schema(self):
+        return {'type': ['string'], 'pattern': self._PATTERN}
+
 
 class Integer(FieldType):
     @staticmethod
     def coerce(obj, attr, value):
         return int(value)
 
+    def get_schema(self):
+        return {'type': ['integer']}
+
+
+class NonNegativeInteger(FieldType):
+    @staticmethod
+    def coerce(obj, attr, value):
+        v = int(value)
+        if v < 0:
+            raise ValueError(_('Value must be >= 0 for field %s') % attr)
+        return v
+
+    def get_schema(self):
+        return {'type': ['integer'], 'minimum': 0}
+
 
 class Float(FieldType):
     def coerce(self, obj, attr, value):
         return float(value)
 
+    def get_schema(self):
+        return {'type': ['number']}
+
+
+class NonNegativeFloat(FieldType):
+    @staticmethod
+    def coerce(obj, attr, value):
+        v = float(value)
+        if v < 0:
+            raise ValueError(_('Value must be >= 0 for field %s') % attr)
+        return v
+
+    def get_schema(self):
+        return {'type': ['number'], 'minimum': 0}
+
 
 class Boolean(FieldType):
     @staticmethod
     def coerce(obj, attr, value):
         return bool(value)
 
+    def get_schema(self):
+        return {'type': ['boolean']}
+
 
 class FlexibleBoolean(Boolean):
     @staticmethod
@@ -407,6 +474,9 @@ class DateTime(FieldType):
     def from_primitive(self, obj, attr, value):
         return self.coerce(obj, attr, timeutils.parse_isotime(value))
 
+    def get_schema(self):
+        return {'type': ['string'], 'format': 'date-time'}
+
     @staticmethod
     def to_primitive(obj, attr, value):
         return _utils.isotime(value)
@@ -442,6 +512,9 @@ class IPV4Address(IPAddress):
                              {'val': value, 'attr': attr})
         return result
 
+    def get_schema(self):
+        return {'type': ['string'], 'format': 'ipv4'}
+
 
 class IPV6Address(IPAddress):
     @staticmethod
@@ -453,6 +526,24 @@ class IPV6Address(IPAddress):
                              {'val': value, 'attr': attr})
         return result
 
+    def get_schema(self):
+        return {'type': ['string'], 'format': 'ipv6'}
+
+
+class IPV4AndV6Address(IPAddress):
+    @staticmethod
+    def coerce(obj, attr, value):
+        result = IPAddress.coerce(obj, attr, value)
+        if result.version != 4 and result.version != 6:
+            raise ValueError(_('Network "%(val)s" is not valid '
+                               'in field %(attr)s') %
+                             {'val': value, 'attr': attr})
+        return result
+
+    def get_schema(self):
+        return {'oneOf': [IPV4Address().get_schema(),
+                          IPV6Address().get_schema()]}
+
 
 class IPNetwork(IPAddress):
     @staticmethod
@@ -464,6 +555,11 @@ class IPNetwork(IPAddress):
 
 
 class IPV4Network(IPNetwork):
+
+    _PATTERN = (r'^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-'
+                r'9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])(\/([0-9]|[1-2]['
+                r'0-9]|3[0-2]))$')
+
     @staticmethod
     def coerce(obj, attr, value):
         try:
@@ -471,8 +567,16 @@ class IPV4Network(IPNetwork):
         except netaddr.AddrFormatError as e:
             raise ValueError(six.text_type(e))
 
+    def get_schema(self):
+        return {'type': ['string'], 'pattern': self._PATTERN}
+
 
 class IPV6Network(IPNetwork):
+
+    def __init__(self, *args, **kwargs):
+        super(IPV6Network, self).__init__(*args, **kwargs)
+        self._PATTERN = self._create_pattern()
+
     @staticmethod
     def coerce(obj, attr, value):
         try:
@@ -480,6 +584,49 @@ class IPV6Network(IPNetwork):
         except netaddr.AddrFormatError as e:
             raise ValueError(six.text_type(e))
 
+    def _create_pattern(self):
+        ipv6seg = '[0-9a-fA-F]{1,4}'
+        ipv4seg = '(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])'
+
+        return (
+            # Pattern based on answer to
+            # http://stackoverflow.com/questions/53497/regular-expression-that-matches-valid-ipv6-addresses
+            '^'
+            # 1:2:3:4:5:6:7:8
+            '(' + ipv6seg + ':){7,7}' + ipv6seg + '|'
+            # 1:: 1:2:3:4:5:6:7::
+            '(' + ipv6seg + ':){1,7}:|'
+            # 1::8 1:2:3:4:5:6::8 1:2:3:4:5:6::8
+            '(' + ipv6seg + ':){1,6}:' + ipv6seg + '|'
+            # 1::7:8 1:2:3:4:5::7:8 1:2:3:4:5::8
+            '(' + ipv6seg + ':){1,5}(:' + ipv6seg + '){1,2}|'
+            # 1::6:7:8 1:2:3:4::6:7:8 1:2:3:4::8
+            '(' + ipv6seg + ':){1,4}(:' + ipv6seg + '){1,3}|'
+            # 1::5:6:7:8 1:2:3::5:6:7:8 1:2:3::8
+            '(' + ipv6seg + ':){1,3}(:' + ipv6seg + '){1,4}|'
+            # 1::4:5:6:7:8 1:2::4:5:6:7:8 1:2::8
+            '(' + ipv6seg + ':){1,2}(:' + ipv6seg + '){1,5}|' +
+            # 1::3:4:5:6:7:8 1::3:4:5:6:7:8 1::8
+            ipv6seg + ':((:' + ipv6seg + '){1,6})|'
+            # ::2:3:4:5:6:7:8 ::2:3:4:5:6:7:8 ::8 ::
+            ':((:' + ipv6seg + '){1,7}|:)|'
+            # fe80::7:8%eth0 fe80::7:8%1
+            'fe80:(:[0-9a-fA-F]{0,4}){0,4}%[0-9a-zA-Z]{1,}|'
+            # ::255.255.255.255 ::ffff:255.255.255.255 ::ffff:0:255.255.255.255
+            '::(ffff(:0{1,4}){0,1}:){0,1}'
+            '(' + ipv4seg + '\.){3,3}' +
+            ipv4seg + '|'
+            # 2001:db8:3:4::192.0.2.33 64:ff9b::192.0.2.33
+            '(' + ipv6seg + ':){1,4}:'
+            '(' + ipv4seg + '\.){3,3}' +
+            ipv4seg +
+            # /128
+            '(\/(d|dd|1[0-1]d|12[0-8]))$'
+            )
+
+    def get_schema(self):
+        return {'type': ['string'], 'pattern': self._PATTERN}
+
 
 class CompoundFieldType(FieldType):
     def __init__(self, element_type, **field_args):
@@ -507,6 +654,9 @@ class List(CompoundFieldType):
         return '[%s]' % (
             ','.join([self._element_type.stringify(x) for x in value]))
 
+    def get_schema(self):
+        return {'type': ['array'], 'items': self._element_type.get_schema()}
+
 
 class Dict(CompoundFieldType):
     def coerce(self, obj, attr, value):
@@ -538,6 +688,10 @@ class Dict(CompoundFieldType):
             ','.join(['%s=%s' % (key, self._element_type.stringify(val))
                       for key, val in sorted(value.items())]))
 
+    def get_schema(self):
+        return {'type': ['object'],
+                'additionalProperties': self._element_type.get_schema()}
+
 
 class DictProxyField(object):
     """Descriptor allowing us to assign pinning data as a dict of key_types
@@ -594,6 +748,10 @@ class Set(CompoundFieldType):
         return 'set([%s])' % (
             ','.join([self._element_type.stringify(x) for x in value]))
 
+    def get_schema(self):
+        return {'type': ['array'], 'uniqueItems': True,
+                'items': self._element_type.get_schema()}
+
 
 class Object(FieldType):
     def __init__(self, obj_name, subclasses=False, **kwargs):
@@ -627,10 +785,17 @@ class Object(FieldType):
                 # If we're not dealing with an object, it's probably a
                 # primitive so get it's type for the message below.
                 obj_name = type(value).__name__
+            obj_mod = ''
+            if hasattr(obj, '__module__'):
+                obj_mod = ''.join([obj.__module__, '.'])
+            val_mod = ''
+            if hasattr(value, '__module__'):
+                val_mod = ''.join([value.__module__, '.'])
             raise ValueError(_('An object of type %(type)s is required '
                                'in field %(attr)s, not a %(valtype)s') %
-                             {'type': self._obj_name, 'attr': attr,
-                              'valtype': obj_name})
+                             {'type': ''.join([obj_mod, self._obj_name]),
+                              'attr': attr, 'valtype': ''.join([val_mod,
+                                                                obj_name])})
         return value
 
     @staticmethod
@@ -662,6 +827,54 @@ class Object(FieldType):
 
         return '%s%s' % (self._obj_name, ident)
 
+    def get_schema(self):
+        from oslo_versionedobjects import base as obj_base
+        obj_classes = obj_base.VersionedObjectRegistry.obj_classes()
+        if self._obj_name in obj_classes:
+            cls = obj_classes[self._obj_name][0]
+            namespace_key = cls._obj_primitive_key('namespace')
+            name_key = cls._obj_primitive_key('name')
+            version_key = cls._obj_primitive_key('version')
+            data_key = cls._obj_primitive_key('data')
+            changes_key = cls._obj_primitive_key('changes')
+            field_schemas = {key: field.get_schema()
+                             for key, field in cls.fields.items()}
+            required_fields = [key for key, field in sorted(cls.fields.items())
+                               if not field.nullable]
+            schema = {
+                'type': 'object',
+                'properties': {
+                    namespace_key: {
+                        'type': 'string'
+                    },
+                    name_key: {
+                        'type': 'string'
+                    },
+                    version_key: {
+                        'type': 'string'
+                    },
+                    changes_key: {
+                        'type': 'array',
+                        'items': {
+                            'type': 'string'
+                        }
+                    },
+                    data_key: {
+                        'type': 'object',
+                        'description': 'fields of %s' % self._obj_name,
+                        'properties': field_schemas,
+                    },
+                },
+                'required': [namespace_key, name_key, version_key, data_key]
+            }
+
+            if required_fields:
+                schema['properties'][data_key]['required'] = required_fields
+
+            return schema
+        else:
+            raise exception.UnsupportedObjectError(objtype=self._obj_name)
+
 
 class AutoTypedField(Field):
     AUTO_TYPE = None
@@ -860,10 +1073,18 @@ class IntegerField(AutoTypedField):
     AUTO_TYPE = Integer()
 
 
+class NonNegativeIntegerField(AutoTypedField):
+    AUTO_TYPE = NonNegativeInteger()
+
+
 class FloatField(AutoTypedField):
     AUTO_TYPE = Float()
 
 
+class NonNegativeFloatField(AutoTypedField):
+    AUTO_TYPE = NonNegativeFloat()
+
+
 # This is a strict interpretation of boolean
 # values using Python's semantics for truth/falsehood
 class BooleanField(AutoTypedField):
@@ -930,6 +1151,10 @@ class ListOfSetsOfIntegersField(AutoTypedField):
     AUTO_TYPE = List(Set(Integer()))
 
 
+class ListOfIntegersField(AutoTypedField):
+    AUTO_TYPE = List(Integer())
+
+
 class ListOfDictOfNullableStringsField(AutoTypedField):
     AUTO_TYPE = List(Dict(String(), nullable=True))
 
@@ -960,6 +1185,10 @@ class IPV6AddressField(AutoTypedField):
     AUTO_TYPE = IPV6Address()
 
 
+class IPV4AndV6AddressField(AutoTypedField):
+    AUTO_TYPE = IPV4AndV6Address()
+
+
 class IPNetworkField(AutoTypedField):
     AUTO_TYPE = IPNetwork()
 
