@@ -217,6 +217,17 @@ class TestMACAddress(TestField):
         self.to_primitive_values = self.coerce_good_values[0:1]
         self.from_primitive_values = self.coerce_good_values[0:1]
 
+    def test_get_schema(self):
+        schema = self.field.get_schema()
+        self.assertEqual(['string'], schema['type'])
+        self.assertEqual(False, schema['readonly'])
+        pattern = schema['pattern']
+        for _, valid_val in self.coerce_good_values:
+            self.assertRegex(valid_val, pattern)
+        invalid_vals = [x for x in self.coerce_bad_values if type(x) == 'str']
+        for invalid_val in invalid_vals:
+            self.assertNotRegex(invalid_val, pattern)
+
 
 class TestPCIAddress(TestField):
     def setUp(self):
@@ -238,12 +249,21 @@ class TestPCIAddress(TestField):
         self.to_primitive_values = self.coerce_good_values[0:1]
         self.from_primitive_values = self.coerce_good_values[0:1]
 
+    def test_get_schema(self):
+        schema = self.field.get_schema()
+        self.assertEqual(['string'], schema['type'])
+        self.assertEqual(False, schema['readonly'])
+        pattern = schema['pattern']
+        for _, valid_val in self.coerce_good_values:
+            self.assertRegex(valid_val, pattern)
+        invalid_vals = [x for x in self.coerce_bad_values if type(x) == 'str']
+        for invalid_val in invalid_vals:
+            self.assertNotRegex(invalid_val, pattern)
+
 
 class TestUUID(TestField):
     def setUp(self):
         super(TestUUID, self).setUp()
-
-    def test_validation_enabled(self):
         self.field = fields.UUIDField()
         self.coerce_good_values = [
             ('da66a411-af0e-4829-9b67-475017ddd152',
@@ -267,14 +287,27 @@ class TestUUID(TestField):
             ('1', '1'),
             (1, '1')
         ]
-
         self.to_primitive_values = self.coerce_good_values[0:1]
         self.from_primitive_values = self.coerce_good_values[0:1]
 
+    def test_validation_enabled(self):
+
         self.test_coerce_good_values()
         self.test_from_primitive()
         self.test_to_primitive()
 
+    def test_get_schema(self):
+        field = fields.UUIDField()
+        schema = field.get_schema()
+        self.assertEqual(['string'], schema['type'])
+        self.assertEqual(False, schema['readonly'])
+        pattern = schema['pattern']
+        for _, valid_val in self.coerce_good_values[:4]:
+            self.assertRegex(valid_val, pattern)
+        invalid_vals = [x for x in self.coerce_bad_values if type(x) == 'str']
+        for invalid_val in invalid_vals:
+            self.assertNotRegex(invalid_val, pattern)
+
 
 class TestBaseEnum(TestField):
     def setUp(self):
@@ -444,6 +477,20 @@ class TestInteger(TestField):
                          self.field.get_schema())
 
 
+class TestNonNegativeInteger(TestField):
+    def setUp(self):
+        super(TestNonNegativeInteger, self).setUp()
+        self.field = fields.NonNegativeIntegerField()
+        self.coerce_good_values = [(1, 1), ('1', 1)]
+        self.coerce_bad_values = ['-2', '4.2', 'foo', None]
+        self.to_primitive_values = self.coerce_good_values[0:1]
+        self.from_primitive_values = self.coerce_good_values[0:1]
+
+    def test_get_schema(self):
+        self.assertEqual({'type': ['integer'], 'readonly': False,
+                          'minimum': 0}, self.field.get_schema())
+
+
 class TestFloat(TestField):
     def setUp(self):
         super(TestFloat, self).setUp()
@@ -461,6 +508,20 @@ class TestFloat(TestField):
                          self.field.get_schema())
 
 
+class TestNonNegativeFloat(TestField):
+    def setUp(self):
+        super(TestNonNegativeFloat, self).setUp()
+        self.field = fields.NonNegativeFloatField()
+        self.coerce_good_values = [(1.1, 1.1), ('1.1', 1.1)]
+        self.coerce_bad_values = ['-4.2', 'foo', None]
+        self.to_primitive_values = self.coerce_good_values[0:1]
+        self.from_primitive_values = self.coerce_good_values[0:1]
+
+    def test_get_schema(self):
+        self.assertEqual({'type': ['number'], 'readonly': False,
+                          'minimum': 0}, self.field.get_schema())
+
+
 class TestBoolean(TestField):
     def setUp(self):
         super(TestField, self).setUp()
@@ -514,6 +575,11 @@ class TestDateTime(TestField):
                 datetime.datetime(1955, 11, 5, 18, 0, 0,
                                   tzinfo=iso8601.iso8601.Utc())))
 
+    def test_get_schema(self):
+        self.assertEqual({'type': ['string'], 'format': 'date-time',
+                          'readonly': False},
+                         self.field.get_schema())
+
 
 class TestDateTimeNoTzinfo(TestField):
     def setUp(self):
@@ -551,6 +617,13 @@ class TestDict(TestField):
     def test_stringify(self):
         self.assertEqual("{key=val}", self.field.stringify({'key': 'val'}))
 
+    def test_get_schema(self):
+        self.assertEqual({'type': ['object'],
+                          'additionalProperties': {'readonly': False,
+                                                   'type': ['foo']},
+                          'readonly': False},
+                         self.field.get_schema())
+
 
 class TestDictOfStrings(TestField):
     def setUp(self):
@@ -710,6 +783,12 @@ class TestSet(TestField):
     def test_stringify(self):
         self.assertEqual('set([123])', self.field.stringify(set([123])))
 
+    def test_get_schema(self):
+        self.assertEqual({'type': ['array'], 'uniqueItems': True,
+                          'items': {'type': ['foo'], 'readonly': False},
+                          'readonly': False},
+                         self.field.get_schema())
+
 
 class TestSetOfIntegers(TestField):
     def setUp(self):
@@ -739,6 +818,20 @@ class TestListOfSetsOfIntegers(TestField):
         self.assertEqual('[set([1,2])]', self.field.stringify([set([1, 2])]))
 
 
+class TestListOfIntegers(TestField):
+    def setUp(self):
+        super(TestListOfIntegers, self).setUp()
+        self.field = fields.ListOfIntegersField()
+        self.coerce_good_values = [(['1', 2], [1, 2]),
+                                   ([1, 2], [1, 2])]
+        self.coerce_bad_values = [['foo']]
+        self.to_primitive_values = [([1], [1])]
+        self.from_primitive_values = [([1], [1])]
+
+    def test_stringify(self):
+        self.assertEqual('[[1, 2]]', self.field.stringify([[1, 2]]))
+
+
 class TestLocalMethods(test.TestCase):
     @mock.patch.object(obj_base.LOG, 'exception')
     def test__make_class_properties_setter_value_error(self, mock_log):
@@ -927,6 +1020,31 @@ class TestObject(TestField):
         self.assertEqual('An object of type TestableObject is required '
                          'in field attr, not a list', six.text_type(ex))
 
+    def test_get_schema(self):
+        self.assertEqual(
+            {
+                'properties': {
+                    'versioned_object.changes':
+                        {'items': {'type': 'string'}, 'type': 'array'},
+                    'versioned_object.data': {
+                        'description': 'fields of TestableObject',
+                        'properties':
+                            {'uuid': {'readonly': False, 'type': ['string']}},
+                        'required': ['uuid'],
+                        'type': 'object'},
+                    'versioned_object.name': {'type': 'string'},
+                    'versioned_object.namespace': {'type': 'string'},
+                    'versioned_object.version': {'type': 'string'}
+                },
+                'readonly': False,
+                'required': ['versioned_object.namespace',
+                             'versioned_object.name',
+                             'versioned_object.version',
+                             'versioned_object.data'],
+                'type': 'object'
+            },
+            self.field.get_schema())
+
 
 class TestIPAddress(TestField):
     def setUp(self):
@@ -957,6 +1075,11 @@ class TestIPAddressV4(TestField):
         self.from_primitive_values = [('1.2.3.4',
                                        netaddr.IPAddress('1.2.3.4'))]
 
+    def test_get_schema(self):
+        self.assertEqual({'type': ['string'], 'readonly': False,
+                          'format': 'ipv4'},
+                         self.field.get_schema())
+
 
 class TestIPAddressV6(TestField):
     def setUp(self):
@@ -967,8 +1090,39 @@ class TestIPAddressV6(TestField):
                                     netaddr.IPAddress('::1'))]
         self.coerce_bad_values = ['1.2', 'foo', '1.2.3.4']
         self.to_primitive_values = [(netaddr.IPAddress('::1'), '::1')]
+        self.from_primitive_values = [('::1', netaddr.IPAddress('::1'))]
+
+    def test_get_schema(self):
+        self.assertEqual({'type': ['string'], 'readonly': False,
+                          'format': 'ipv6'},
+                         self.field.get_schema())
+
+
+class TestIPV4AndV6Address(TestField):
+    def setUp(self):
+        super(TestIPV4AndV6Address, self).setUp()
+        self.field = fields.IPV4AndV6Address()
+        self.coerce_good_values = [('::1', netaddr.IPAddress('::1')),
+                                   (netaddr.IPAddress('::1'),
+                                    netaddr.IPAddress('::1')),
+                                   ('1.2.3.4',
+                                    netaddr.IPAddress('1.2.3.4')),
+                                   (netaddr.IPAddress('1.2.3.4'),
+                                    netaddr.IPAddress('1.2.3.4'))]
+        self.coerce_bad_values = ['1-2', 'foo']
+        self.to_primitive_values = [(netaddr.IPAddress('::1'),
+                                     '::1'),
+                                    (netaddr.IPAddress('1.2.3.4'),
+                                     '1.2.3.4')]
         self.from_primitive_values = [('::1',
-                                       netaddr.IPAddress('::1'))]
+                                       netaddr.IPAddress('::1')),
+                                      ('1.2.3.4',
+                                       netaddr.IPAddress('1.2.3.4'))]
+
+    def test_get_schema(self):
+        self.assertEqual({'oneOf': [{'format': 'ipv4', 'type': ['string']},
+                                    {'format': 'ipv6', 'type': ['string']}]},
+                         self.field.get_schema())
 
 
 class TestIPNetwork(TestField):
@@ -998,6 +1152,17 @@ class TestIPV4Network(TestField):
         self.from_primitive_values = [('1.2.3.4/24',
                                        netaddr.IPNetwork('1.2.3.4/24'))]
 
+    def test_get_schema(self):
+        schema = self.field.get_schema()
+        self.assertEqual(['string'], schema['type'])
+        self.assertEqual(False, schema['readonly'])
+        pattern = schema['pattern']
+        for _, valid_val in self.coerce_good_values:
+            self.assertRegex(str(valid_val), pattern)
+        invalid_vals = [x for x in self.coerce_bad_values]
+        for invalid_val in invalid_vals:
+            self.assertNotRegex(str(invalid_val), pattern)
+
 
 class TestIPV6Network(TestField):
     def setUp(self):
@@ -1010,3 +1175,14 @@ class TestIPV6Network(TestField):
         self.to_primitive_values = [(netaddr.IPNetwork('::1/0'), '::1/0')]
         self.from_primitive_values = [('::1/0',
                                        netaddr.IPNetwork('::1/0'))]
+
+    def test_get_schema(self):
+        schema = self.field.get_schema()
+        self.assertEqual(['string'], schema['type'])
+        self.assertEqual(False, schema['readonly'])
+        pattern = schema['pattern']
+        for _, valid_val in self.coerce_good_values:
+            self.assertRegex(str(valid_val), pattern)
+        invalid_vals = [x for x in self.coerce_bad_values]
+        for invalid_val in invalid_vals:
+            self.assertNotRegex(str(invalid_val), pattern)
